/*
 *   Copyright (C) 2001  Andrea Gini
 *
 *   Require JDK 1.3
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Send questions or suggestions to ginian@tiscalinet.it
 *
 */
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

public class FindAndReplaceDialog extends FindDialog {

  private JTextField replaceTextField;
  private JButton replaceButton;
  
  public FindAndReplaceDialog(TextEditor owner) {
    super(owner);
  }
  protected void setupFrame() {
    setTitle("Find");
    setResizable(false);
  }
  protected void setupFrameBehaviour() {
    setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
  }
  protected void setupComponents() {  
    super.setupComponents();
    replaceTextField = new JTextField(15);    
    replaceButton = new JButton(new ImageIcon("Replace24.gif"));    
  }
  protected Box createMainBox() {
    Box box = super.createMainBox();
      Box replaceBox = Box.createHorizontalBox();
      replaceBox.add(new JLabel("Replace with:"));
      replaceBox.add(replaceTextField);
      replaceBox.add(replaceButton);
               
    box.add(replaceBox);

    return box;
  }
  protected Border createTitledBorder() {
    return BorderFactory.createTitledBorder("Find And Replace");  
  }
  protected void registerListeners() {
    super.registerListeners();
    ActionListener replaceListener = new ReplaceButtonListener();
    replaceButton.addActionListener(replaceListener);
    replaceTextField.addActionListener(replaceListener);
  }
  protected void enableFind() {
    super.enableFind();
    replaceButton.setEnabled(true);
    replaceTextField.setEnabled(true);
  }
  protected void disableFind() {
    super.disableFind();
    replaceButton.setEnabled(false);
    replaceTextField.setEnabled(false);     
  }
  private int promptReplaceAll() {
        Object[] options = { "Replace", "Replace All", "Next", "CANCEL" };
        return JOptionPane.showOptionDialog(this, "Replace this occourrence of "+
                    getFindTextField().getText(),
                    "Replace?", 
                    JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE,
                    null, options, options[0]);  
  }
  public void replace() {
    boolean prompt = true;
    while(true)
      try {
        getTarget().goToStringFromActualOffset(getFindTextField().getText());
        
        if(prompt) {
          int returnVal = promptReplaceAll();
      
          if(returnVal==1)
            prompt=false;
          else if(returnVal==2) {
            getTarget().goToStringFromNextOffset(getFindTextField().getText());
            continue;
          }
          else if(returnVal==3)
            break;
        }
        
        getTarget().replaceSelection(replaceTextField.getText());
      }
      catch(NotFoundException ex) {
        JOptionPane.showMessageDialog(this,
          "Finished!",
          "Finished!",
          JOptionPane.WARNING_MESSAGE);
        break;  
      }
  }
  class ReplaceButtonListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      replace();
    }
  }      
  public static void main(String argv[]) {
    JDialog f = new FindAndReplaceDialog(null);
    f.setVisible(true);
  }
}