/*
 *   Copyright (C) 2001  Andrea Gini
 *
 *   Require JDK 1.3
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Send questions or suggestions to ginian@tiscalinet.it
 *
 */

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;

public class FindDialog extends JDialog {

  private TextEditor target;
  private JButton nextButton;

  private JButton cancelButton;
  private JTextField findTextField;
  
  public FindDialog(TextEditor owner) {
    super(owner);
    this.target = owner;
    setupFrame();
    setupFrameBehaviour();
    setupComponents();
    buildGui();
    registerListeners();    
    disableFind();
  }
  protected TextEditor getTarget() {
    return target;
  }
  protected void setupFrame() {
    setTitle("Find");
    setResizable(false);
  }
  protected void setupFrameBehaviour() {
    setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
  }
  protected void setupComponents() {  
    findTextField = new JTextField(15);
    nextButton = new JButton(new ImageIcon("FindAgain24.gif")); 
    cancelButton = new JButton("Cancel");
  }
  protected JTextField getFindTextField() {
    return findTextField;
  }
  protected Box createMainBox() {
    Box box = Box.createVerticalBox();
      Box findBox = Box.createHorizontalBox();
      findBox.add(new JLabel("Find:                 "));
      findBox.add(findTextField);
      findBox.add(nextButton);
               
    box.add(Box.createVerticalStrut(20));
    box.add(findBox);
    return box;
  }
  protected Border createTitledBorder() {
    return BorderFactory.createTitledBorder("Find");
  }
  protected void buildGui() {
    Box mainBox = createMainBox();
    mainBox.add(Box.createVerticalStrut(50));  
    
    JPanel southPanel = new JPanel();
    southPanel.setLayout(new FlowLayout(FlowLayout.RIGHT));
    southPanel.add(cancelButton);

    JPanel contentPane = new JPanel();
    contentPane.setBorder(createTitledBorder());
    contentPane.setLayout(new BorderLayout());
    contentPane.add(mainBox);
    contentPane.add(BorderLayout.SOUTH,southPanel);
    setContentPane(contentPane);
    pack();
  }
  protected void registerListeners() {
    ActionListener nextListener = new NextButtonListener();
    nextButton.addActionListener(nextListener);
    findTextField.addActionListener(nextListener);

    cancelButton.addActionListener(new CancelButtonListener());

    findTextField.getDocument().addDocumentListener(new TextFieldDocumentListener());
  }
  protected void enableFind() {
    findTextField.setForeground(Color.black);
    nextButton.setEnabled(true);
  }
  protected void disableFind() {
    findTextField.setForeground(Color.red);
    nextButton.setEnabled(false);
  }
  public void findCurrent() {
    try {
      target.goToStringFromActualOffset(findTextField.getText());
      enableFind();
    }
    catch(NotFoundException ex) {
      disableFind();
    }  
  }
  public void findNext() {
    try {
      target.goToStringFromNextOffset(findTextField.getText());
    }

  catch(NotFoundException ex) {
      JOptionPane.showMessageDialog(this,
        "Not Found",
        "String Not Found",
        JOptionPane.WARNING_MESSAGE);      
    }  
  }

  class NextButtonListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      findNext();
    }
  }

  class CancelButtonListener implements ActionListener {
    public void actionPerformed(ActionEvent e) {
      setVisible(false);
    }
  }  

  class TextFieldDocumentListener implements DocumentListener {
    public void changedUpdate(DocumentEvent e) {}
    public void insertUpdate(DocumentEvent e) {
      findCurrent();
    }
    public void removeUpdate(DocumentEvent e) {
      findCurrent();
    }
  }  

public static void main(String argv[]) {
  FindDialog f = new FindDialog(null);
  f.setVisible(true);
}
}