/*
 *   Copyright (C) 2001  Andrea Gini
 *
 *   Require JDK 1.3
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *
 *   Send questions or suggestions to ginian@tiscalinet.it
 *
 */

import java.util.*;
import javax.swing.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

public class TextEditor extends JFrame implements ActionListener {

  private JTextComponent editor;
  private JFileChooser fileChooser;
  private FindDialog findDialog;
    
  protected JMenuItem OpenMenuItem;
  protected JMenuItem SaveMenuItem;
  protected JMenuItem FindMenuItem; 
  protected JMenuItem CutMenuItem;
  protected JMenuItem CopyMenuItem;
  protected JMenuItem PasteMenuItem; 
    
  protected AbstractButton OpenButton; 
  protected AbstractButton SaveButton;
  protected AbstractButton FindButton;
  protected AbstractButton CutButton;
  protected AbstractButton CopyButton;
  protected AbstractButton PasteButton;

  private Action[] actions;
  private Hashtable commands;
  
  public TextEditor() {
    super();
    setupFrame();
    setupFrameBehaviour();
    setupComponents();
    buildGui();
    registerListeners();    
  }
  protected void setupFrame() {
    setTitle("TextEditor");
    setSize(300,300);
  }
  protected void setupFrameBehaviour() {
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);  
  }
  protected void setupComponents() {
    editor = createEditor();
    fileChooser = createFileChooser();
    findDialog = new FindDialog(this);
    buildButtons();
    buildMenuItems();    
  }
  protected JTextComponent createEditor() {
    return new JTextArea();
  }
  protected JFileChooser createFileChooser() {
    return new JFileChooser();
  }
  public JTextComponent getEditor() {
    return editor;
  }
  protected JFileChooser getFileChooser() {
    return fileChooser;
  }

  protected void buildMenuItems() {
    OpenMenuItem = createMenuItem("Open",new ImageIcon("Open24.gif"));
    SaveMenuItem = createMenuItem("Save",new ImageIcon("Save24.gif"));
    FindMenuItem = createMenuItem("Find",new ImageIcon("Find24.gif"));
    CutMenuItem = createMenuItem("Cut",new ImageIcon("Cut24.gif"));
    CopyMenuItem = createMenuItem("Copy",new ImageIcon("Copy24.gif"));
    PasteMenuItem = createMenuItem("Paste",new ImageIcon("Paste24.gif"));

  }
  protected JMenuItem createMenuItem(String name,Icon icon) {
    return new JMenuItem(name,icon);
  }
  protected JMenuItem createMenuItem(String name) {
    return new JMenuItem(name);
  }
  protected void buildButtons() {
    OpenButton = createToolbarButton(new ImageIcon("Open24.gif"));
    SaveButton = createToolbarButton(new ImageIcon("Save24.gif"));
    FindButton = createToolbarButton(new ImageIcon("Find24.gif"));
    CutButton = createToolbarButton(new ImageIcon("Cut24.gif"));
    CopyButton = createToolbarButton(new ImageIcon("Copy24.gif"));
    PasteButton = createToolbarButton(new ImageIcon("Paste24.gif"));
  }
  protected AbstractButton createToolbarButton(Icon icon) {
    return new JButton(icon);
  }

  protected void buildGui() {
    JMenuBar menuBar = createMenuBar();
    JToolBar toolBar = createToolBar();
   
    getContentPane().setLayout(new BorderLayout());
    getContentPane().add(BorderLayout.NORTH,toolBar);
    getContentPane().add(BorderLayout.CENTER,new JScrollPane(getEditor()));
    setJMenuBar(menuBar);
  }
  protected JMenuBar createMenuBar() {
    JMenu menu = new JMenu("Menu");
      menu.add(OpenMenuItem);
      menu.add(SaveMenuItem);
      menu.addSeparator();
      menu.add(FindMenuItem);
      menu.addSeparator();      
      menu.add(CutMenuItem);
      menu.add(CopyMenuItem);
      menu.add(PasteMenuItem);
    JMenuBar menuBar = new JMenuBar();
    menuBar.add(menu);
    
    return menuBar;
  }
  protected JToolBar createToolBar() {
    JToolBar toolBar = new JToolBar();
    toolBar.add(OpenButton);
    toolBar.add(SaveButton);
    toolBar.addSeparator();
    toolBar.add(FindButton);
    toolBar.addSeparator();    
    toolBar.add(CutButton);
    toolBar.add(CopyButton);
    toolBar.add(PasteButton);  
    return toolBar;
  }
  protected void registerListeners() {
    OpenMenuItem.addActionListener(this);
    SaveMenuItem.addActionListener(this);
    FindMenuItem.addActionListener(this);
    CutMenuItem.addActionListener(this);
    CopyMenuItem.addActionListener(this);
    PasteMenuItem.addActionListener(this);
   
    OpenButton.addActionListener(this);
    SaveButton.addActionListener(this);
    FindButton.addActionListener(this);
    CutButton.addActionListener(this);
    CopyButton.addActionListener(this);
    PasteButton.addActionListener(this);
  }

  public void promptOpen() {
    int response = getFileChooser().showOpenDialog(this);
    if(response==JFileChooser.APPROVE_OPTION) {
      try {
        open(getFileChooser().getSelectedFile());
      }
      catch(IllegalArgumentException e) {
        JOptionPane.showMessageDialog(this,
          "Tipo di file non consentito",
          "Wrong Filetype",
          JOptionPane.WARNING_MESSAGE);
      }      
      catch(IOException e) {
        JOptionPane.showMessageDialog(this,
          "Errore di Input Output",
          "I/O Error",
          JOptionPane.WARNING_MESSAGE);
      }
    }  
  }
  public void open(File f) throws IOException,IllegalArgumentException {
    if(isValidFiletype(f)) {
      Reader in = new FileReader(f);
      // la riga seguente puo' generare una IOException
      editor.read(in,null);
    }
    else
      throw new IllegalArgumentException();
  }
  protected boolean isValidFiletype(File f) {
    return f.getName().toLowerCase().endsWith(".txt");
  }
  public void promptSaveAs() {     int response = getFileChooser().showSaveDialog(this);
    if(response==JFileChooser.APPROVE_OPTION) {
      try {
        save(getFileChooser().getSelectedFile());
      }
      catch(IOException e) {
        JOptionPane.showMessageDialog(this,
          "Errore di Input Output",
          "I/O Error",
          JOptionPane.WARNING_MESSAGE);    
      }
    }  
  }
  public void save(File f) throws IOException{
    Writer out = new FileWriter(f);
    // la riga seguente puo' generare una IOException
    editor.write(out);
  }
  public void cutSelection() {
    getEditor().cut();
  }
  public void copySelection() {
    getEditor().copy();
  }
  public void pasteSelection() {
    getEditor().paste();
  }
  public void promptFind() {
    findDialog.setVisible(true);
  }
  /**
   * Cerca la stringa indicata dal parametro all'interno del testo dell'editor,
   * partendo dalla posizione attuale del cursore. La ricerca viene svolta dall'alto
   * al basso in modo circolare.
   * Se la trova, seleziona la prima occorrenza, altrimenti lancia
   * una NotFoundException.
   */
  public void goToStringFromActualOffset(String toFind) throws NotFoundException {
    int fromIndex = getEditor().getSelectionStart();
    try {
      goToString(toFind,fromIndex);
    }
    catch(BadLocationException ble) {}
  }
  /**
   * Cerca la stringa indicata dal parametro all'interno del testo dell'editor,
   * partendo dalla posizione del cursore successiva a quella attuale, o dalla
   * prima se il cursore si trovava in ultima posizione. La ricerca viene svolta dall'alto
   * al basso in modo circolare.
   * Se la trova, seleziona la prima occorrenza, altrimenti lancia
   * una NotFoundException.
   */  
  public void goToStringFromNextOffset(String toFind) throws NotFoundException {
    int fromIndex = getEditor().getSelectionStart();
    int length = getEditor().getDocument().getLength();
    try { 
      goToString(toFind,(fromIndex+1)%length);
    }
    catch(BadLocationException ble) {}    
  }
  /**
   * Cerca la stringa indicata dal parametro all'interno del testo dell'editor,
   * partendo dalla posizione del cursore specificata dal parametro. La ricerca
   * viene svolta dall'alto al basso in modo circolare, ossia prosegue prima dall'offset
   * specificato fino alla fine del documento, quindi prova dalla prima posizione
   * fino all'offset.
   * Se la trova, seleziona la prima occorrenza, altrimenti lancia
   * una NotFoundException. Se il parametro punta ad un indice
   * non corretto, viene sollevata una BadLocationException.
   */  
  public void goToString(String toFind,int startOffset) throws NotFoundException,BadLocationException {
    int endOffset = getEditor().getDocument().getLength();
    try {
      goToString(toFind,startOffset,endOffset); 
    }
    catch(NotFoundException nfe) {
      goToString(toFind,0,startOffset);        
    }
  }
  /**
   * Questo metodo viene utilizzato da goToString(String toFind,int startOffset)
   * ed effettua la ricerca vera e propria. La ricerca viene effettuata tra gli
   * offset indicati dai parametri; se questi non sono corretti viene lanciata
   * una BadLocationException
   */
  public void goToString(String toFind,int startOffset,int endOffset) throws NotFoundException,BadLocationException {
    if(toFind.length()==0)
      throw new NotFoundException();

    int length = endOffset-startOffset;
    String text = getEditor().getDocument().getText(startOffset,length);
                                          // throws BadLocationException
    int index = text.indexOf(toFind);
    
    if(index!=-1) {
      getEditor().requestFocus();
      getEditor().select(startOffset+index,startOffset+index+toFind.length());
      return;
    }
    else  
      throw new NotFoundException();      
  }    
  public void actionPerformed(ActionEvent ae) {
    if(ae.getSource().equals(OpenButton) || ae.getSource().equals(OpenMenuItem)) {
      promptOpen();
    }
    else if(ae.getSource().equals(SaveButton) || ae.getSource().equals(SaveMenuItem)) {
      promptSaveAs();
    }
    else if(ae.getSource().equals(FindButton) || ae.getSource().equals(FindMenuItem)) {
      promptFind();
    }
    else if(ae.getSource().equals(CutButton) || ae.getSource().equals(CutMenuItem)) {
      cutSelection();
    }
    else if(ae.getSource().equals(CopyButton) || ae.getSource().equals(CopyMenuItem)) {
      copySelection();
    }
    else if(ae.getSource().equals(PasteButton) || ae.getSource().equals(PasteMenuItem)) {
      pasteSelection();
    }
  }
  public static void main(String argv[]) {
    TextEditor t = new TextEditor();
    if(argv.length==1) {
      File f = new File(argv[0]);
      try {
        t.open(f);
      }
      catch(IllegalArgumentException e) {
        System.out.println("Tipo di file non consentito");
        System.exit(1);
      }    
      catch(IOException e) {
        System.out.println("Errore di I/O in lettura");
        System.exit(2);
      }
    }
    t.setVisible(true);
  }
}